/* $RCSfile: RoseImport.h,v $
 * $Revision: 1.4 $ $Date: 2005/09/20 15:34:21 $
 * Auth: Jochen Fritz (jfritz@steptools.com)
 * 
 * 	Copyright (c) 1991-2006 by 
 * 	STEP Tools Inc., Troy, New York
 * 	All Rights Reserved
 * 
 * 	This software is furnished under a license and may be used and
 * 	copied only in accordance with the terms of such license and with
 * 	the inclusion of the above copyright notice.  This software and
 * 	accompanying written materials or any other copies thereof may
 * 	not be provided or otherwise made available to any other person.
 * 	No title to or ownership of the software is hereby transferred.
 * 
 * 		----------------------------------------
 */

#ifndef ROSE_IMPORT_H
#define ROSE_IMPORT_H

#include "rose.h"

declare(Dictionary,RoseDomain);

class ImportManager;

class RoseImport {
    RoseDesign * f_target;
    RoseDesign * f_schema;
    Dictionary(RoseDomain) substitutes;

    /* This points to the first object in a liked list.  The next object
     * is found by getting the appropiate ImportManager and getting the
     * nextImportManager there, and so on.
     */
    RoseObject * imported_objects;

    ImportManager * getManager (RoseObject * obj);
    void removeManager (RoseObject * obj);
    //void flushForwardRefs(RoseObject * obj, ImportManager * mgr);
public:

    /* Constructor.  Specify the target design and optional schema */
    RoseImport (RoseDesign * target, RoseDesign * schema = NULL);
    RoseImport (RoseDesign * target, char * schema_name);
    RoseImport ();

    virtual ~RoseImport();
    
    void target_design (RoseDesign * des) {f_target = des;}
    RoseDesign * target_design ()   {return f_target;}
    
    void target_schema (const char *);
    void target_schema (RoseDesign *sch ) {f_schema = sch;}
    RoseDesign * target_schema ()         {return f_schema;}

    void add_type_substitute (const char * as_is, const char * to_be);
    void add_type_substitute (const char * as_is, RoseDomain * to_be);
    Dictionary(RoseDomain) * get_type_substitutes();
    
    RoseObject * import (RoseObject * src, RoseDomain * dest=0); /* shallow */
    
    RoseObject * import_deep (RoseObject * src, unsigned depth = 0,
			      RoseDomain * dest =0);    
    
    /* matches type exactly.  No isa done */
    void import_type (RoseDesign * src_des,
		      const char * as_is, const char * to_be=NULL);
    
    void import_type_deep (RoseDesign * src,
			   const char * as_is, const char * to_be=NULL, 
			   unsigned depth = 0);

    void import_all_matching(RoseDesign * src);


    /* Release the associations (i.e. managers) and begin a new transaction */
    void reset();

    /* Batch update any forward references.  NULL out any attributes that
     * point outside the target design.  This operation will preserve the
     * assiciations, so this simply forces the dangling pointers to be updated 
     */ 
    void flush();
   
    /* Get the unimported referenced objects */
    RoseAggregate * get_uncopied_references (RoseAggregate * agg);

private:
    
    void addForwardReference (RoseObject *val, RoseObject * dest,
			      RoseAttribute * destatt, unsigned index);
    
protected:

    /* The schema we are actually using */
    RoseDesign * use_schema ();
    
    void set_association (RoseObject * src, RoseObject * dest);
    RoseObject * get_association (RoseObject * src);
    void clear_association (RoseObject * src);
    


    RoseBoolean putPrimitive
    (RoseObject * dest, RoseAttribute * destatt, unsigned destindex,
     RoseObject * src,  RoseAttribute * srcatt,  unsigned srcidx);
    
    RoseBoolean putObject (RoseObject * obj, RoseAttribute * att,
			   unsigned index, RoseObject * value);
    
    RoseObject * copyAggregate (RoseObject * owner, RoseAttribute *att,
				RoseObject * agg, unsigned depth);

    RoseBoolean importAttribute
    (RoseObject * dest, RoseAttribute * destatt, unsigned destidx,
     RoseObject * src,  RoseAttribute * srcatt,  unsigned srcidx,
     unsigned depth);
    

/* The virtual functions below can be implemented to customize the behavior
 * of the import operation.
 */
    

    /* Callback to allow applications to create the target object.  By default
     * this does targetDesign()->pnewInstance(srcdom->name());
     *
     * It can reuse an existing object, or perform name/domain mapping.
     * The populated parameter should be set to ROSE_TRUE if the object is
     * already populated.
     */
    virtual RoseObject * newObject (RoseObject * srcobj,
				    RoseBoolean & populated,
				    RoseDomain * requested = NULL);

    virtual RoseObject * newAggregate (RoseObject * owner, RoseAttribute * att,
				       RoseObject * srcagg,
				       RoseBoolean & populated);    

    /* This function maps the attribute from the source object to the
     * destination object.  The source object is passed in in case something
     * weird needs to be done
     */
    virtual RoseAttribute * getTargetAttribute (RoseObject * dest,
						RoseAttribute * srcatt);
    
    /* Copy the attributes from source to destination.
     * This can be overridden if you want to populate only a few attributes.
     * This method should call populateAttribte() for every attribute that
     * needs to be copied.
     */
    virtual void populateEntity (RoseObject * dest, RoseObject * src,
				 unsigned depth);

    /* Copy a single attribute -- possibly recursively */
    virtual void populateAttribute (RoseObject * dest, RoseObject * src,
				    RoseAttribute * srcatt, unsigned depth);

    /* Copy a single aggregate element -- possibly recursively */
    virtual void appendElement (RoseObject * dest, RoseObject * src,
				unsigned index, unsigned depth);
    
};


/******************************
 * Usage Examples:


import_stuff (RoseDesign * des);
{
    RoseDesign * targ = pnew RoseDesign ("out", "config_control_design");
    RoseImport imp(targ);

    imp.map_type ("applied_person_and_organization_assignment",
		  "cc_design_person_and_organization_assignment");

    imp.import_all (des);
    imp.flush();
    
    return targ;
}


import_object (List(RoseObject) * obj);
{
    RoseDesign * targ = pnew RoseDesign ("out", "config_control_design");
    RoseImport imp(targ);

    for (unsigned i=0; i<sz; i++) {
	imp.import_deep (objs[i]);
    }

    imp.flush();
}


import_simple (RoseObject * o1, RoseObject *o2);
{
    RoseDesign * targ = pnew RoseDesign ("out", "config_control_design");
    RoseImport imp(targ);

    imp.import(o1);
    imp.import(o2);

    imp.flush();
}
*/

#endif
