/* $RCSfile: rose_refcnt.h,v $
 * $Revision: 1.3 $ $Date: 2005/08/26 18:10:26 $
 * Auth: Dave Loffredo (loffredo@steptools.com)
 * 
 * 	Copyright (c) 1991-2006 by 
 * 	STEP Tools Inc., Troy, New York
 * 	All Rights Reserved
 * 
 * 	This software is furnished under a license and may be used and
 * 	copied only in accordance with the terms of such license and with
 * 	the inclusion of the above copyright notice.  This software and
 * 	accompanying written materials or any other copies thereof may
 * 	not be provided or otherwise made available to any other person.
 * 	No title to or ownership of the software is hereby transferred.
 * 
 * 		----------------------------------------
 */
#ifndef ROSE_REFCNT_H
#define ROSE_REFCNT_H

// Reference counting for detecting unreferenced objects and
// performing clean deletes of nested objects
//

// build reference counts and delete them afterwards.  The reference
// counts are kept in a special manager.
void rose_compute_refcount(RoseDesign*);
void rose_release_refcount(RoseDesign*);

// return current count
unsigned long rose_refcount(RoseObject*);


// increment or decrement count and return new count
unsigned long rose_refcount_inc(RoseObject*);
unsigned long rose_refcount_dec(RoseObject*);
unsigned long rose_refcount_put(RoseObject*, unsigned long);


// For an object, decrements the reference counts of its attributes.
// Only does one level, see below for the recursive version. 

// NOTE, only decrements the attributes, not the original object.
// (presumably it was already unreferences and this is part of a
// cleanup routine)

void rose_refcount_dec_atts(RoseObject*);




// For an object, decrements the reference counts of its attributes.
// If a particular attribute becomes unreferenced because of this, the
// function will decrement its attributes as well, and so on.  If a
// list is passed in, the function will add the objects no longer have
// any references, so that the caller can delete or move them.

// NOTE, only decrements the attributes, not the original object.
// (presumably it was already unreferences and this is part of a
// cleanup routine)

void rose_refcount_recursive_dec_atts(
    RoseObject * obj, 
    ListOfRoseObject * return_now_unrefed = 0
    );

#endif /* ROSE_REFCOUNT_H */
